@@ -1,21 +1,8 @@
-/***********************************************************************
- *
- * Copyright (C) 2009, 2013 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2009 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "board.h"
 
@@ -28,101 +15,135 @@
 #include <QSettings>
 #include <QTimer>
 
+//-----------------------------------------------------------------------------
+
+constexpr int CONNECTAGRAM_GAME_VERSION = 5;
+
+//-----------------------------------------------------------------------------
+
 Board::Board(QObject* parent)
-: QGraphicsScene(parent), m_pattern(0), m_current_word(0), m_hint(0), m_finished(true), m_paused(false) {
+	: QGraphicsScene(parent)
+	, m_pattern(nullptr)
+	, m_current_word(nullptr)
+	, m_hint(nullptr)
+	, m_finished(true)
+	, m_paused(false)
+{
 	QTimer* auto_save = new QTimer(this);
 	auto_save->setInterval(30000);
-	connect(auto_save, SIGNAL(timeout()), this, SLOT(saveGame()));
+	connect(auto_save, &QTimer::timeout, this, &Board::saveGame);
 	auto_save->start();
 
 	m_wordlist = new WordList(this);
+
+	setBackgroundBrush(Qt::white);
 }
 
 //-----------------------------------------------------------------------------
 
-Board::~Board() {
+Board::~Board()
+{
 	cleanUp();
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::check(const QString& original_word, const QString& current_word) {
-	emit wordSolved(original_word, current_word);
+void Board::check(const QString& original_word, const QString& current_word)
+{
+	Q_EMIT wordSolved(original_word, current_word);
 	m_finished = true;
-	foreach (Word* word, m_words) {
+	for (Word* word : std::as_const(m_words)) {
 		m_finished &= word->isCorrect();
 	}
 	if (m_finished) {
 		QSettings().remove("Current/Words");
-		emit finished();
+		Q_EMIT finished();
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::click(const QString& word) {
-	emit wordSelected(word);
+void Board::click(const QString& word)
+{
+	setPaused(true);
+	Q_EMIT wordSelected(word);
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::setCurrentWord(Word* word) {
+void Board::setCurrentWord(Word* word)
+{
 	if (m_current_word) {
 		m_current_word->setHighlight(false);
 	}
-	m_current_word = !m_paused ? word : 0;
+	m_current_word = !m_paused ? word : nullptr;
 	if (m_current_word) {
 		m_current_word->setHighlight(true);
 	}
 	delete m_hint;
-	m_hint = 0;
-	emit hintAvailable(m_current_word != 0);
+	m_hint = nullptr;
+	Q_EMIT hintAvailable(m_current_word);
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::setPaused(bool paused) {
-	if (m_finished) {
-		return;
-	}
-
-	m_paused = paused;
-	for (int x = 0; x < m_cells.count(); ++x) {
-		QList<Cell*>& cells = m_cells[x];
-		for (int y = 0; y < cells.count(); ++y) {
-			Cell* cell = cells[y];
-			if (cell) {
-				cell->letter()->setPaused(paused);
-			}
-		}
-	}
-	if (m_paused) {
-		setCurrentWord(0);
-	}
-	emit pauseChanged();
+QString Board::gameNumber() const
+{
+	return QString("%1%2%3%4%5%6")
+			.arg(CONNECTAGRAM_GAME_VERSION)
+			.arg(m_wordlist->language())
+			.arg(m_pattern->type())
+			.arg(m_pattern->wordCount())
+			.arg(m_pattern->wordLength() - 4, 2, 16, QChar('0'))
+			.arg(m_pattern->seed(), 0, 16);
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::openGame() {
-	cleanUp();
-	emit loading();
-
+bool Board::continueGame()
+{
 	QSettings settings;
-	m_wordlist->setLanguage(settings.value("Current/Language", WordList::defaultLanguage()).toString());
-	m_pattern = Pattern::create(m_wordlist, settings.value("Current/Pattern").toInt());
-	m_pattern->setCount(settings.value("Current/Count").toInt());
-	m_pattern->setLength(settings.value("Current/Length").toInt());
-	m_pattern->setSeed(settings.value("Current/Seed").toInt());
 
-	connect(m_pattern, SIGNAL(generated()), this, SLOT(patternGenerated()));
-	m_pattern->start();
+	if (settings.value("Current/Version").toInt() != CONNECTAGRAM_GAME_VERSION) {
+		settings.remove("Current");
+		return false;
+	}
+
+	settings.beginGroup("Current");
+	const QString language = settings.value("Language", WordList::defaultLanguage()).toString();
+	const int pattern = settings.value("Pattern").toInt();
+	const int count = settings.value("Count").toInt();
+	const int length = settings.value("Length").toInt();
+	const unsigned int seed = settings.value("Seed").toUInt();
+
+	generate(language, pattern, count, length, seed);
+
+	return true;
 }
 
 //-----------------------------------------------------------------------------
 
-bool Board::openGame(const QString& number) {
-	if (!number.startsWith("3")) {
+void Board::newGame(const QString& language, int pattern, int count, int length, unsigned int seed)
+{
+	QSettings settings;
+	settings.remove("Current");
+	settings.beginGroup("Current");
+	settings.setValue("Version", CONNECTAGRAM_GAME_VERSION);
+	settings.setValue("Language", language);
+	settings.setValue("Pattern", pattern);
+	settings.setValue("Count", count);
+	settings.setValue("Length", length);
+	settings.setValue("Seed", seed);
+	settings.setValue("Time", 0);
+
+	generate(language, pattern, count, length, seed);
+}
+
+//-----------------------------------------------------------------------------
+
+bool Board::openGame(const QString& number)
+{
+	if (!number.startsWith(QString::number(CONNECTAGRAM_GAME_VERSION))) {
 		return false;
 	}
 
@@ -167,33 +188,24 @@ bool Board::openGame(const QString& number) {
 	if ((index += 2) == number.length()) {
 		return false;
 	}
-	unsigned int seed = number.mid(index).toInt(&ok, 16);
+	unsigned int seed = number.mid(index).toUInt(&ok, 16);
 	if (!ok) {
 		return false;
 	}
 
 	// Start game
-	cleanUp();
-	emit loading();
-
-	m_wordlist->setLanguage(language);
-	m_pattern = Pattern::create(m_wordlist, pattern);
-	m_pattern->setCount(count);
-	m_pattern->setLength(length);
-	m_pattern->setSeed(seed);
-
-	connect(m_pattern, SIGNAL(generated()), this, SLOT(patternGenerated()));
-	m_pattern->start();
+	newGame(language, pattern, count, length, seed);
 
 	return true;
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::saveGame() {
+void Board::saveGame()
+{
 	if (!m_finished && !m_words.isEmpty()) {
 		QStringList words;
-		foreach (Word* word, m_words) {
+		for (Word* word : std::as_const(m_words)) {
 			words.append(word->toString());
 		}
 		QSettings().setValue("Current/Words", words);
@@ -202,7 +214,8 @@ void Board::saveGame() {
 
 //-----------------------------------------------------------------------------
 
-void Board::showHint() {
+void Board::showHint()
+{
 	if (m_current_word && !mouseGrabberItem()) {
 		delete m_hint;
 		m_hint = m_current_word->hint();
@@ -211,13 +224,48 @@ void Board::showHint() {
 
 //-----------------------------------------------------------------------------
 
-void Board::togglePaused() {
+void Board::setDarkMode(bool enabled)
+{
+	setBackgroundBrush(enabled ? QColor(0x26, 0x26, 0x26) : Qt::white);
+	QSettings().setValue("DarkMode", enabled);
+}
+
+//-----------------------------------------------------------------------------
+
+void Board::setPaused(bool paused)
+{
+	if (m_finished || (paused == m_paused)) {
+		return;
+	}
+
+	m_paused = paused;
+
+	for (const QList<Cell*>& cells : std::as_const(m_cells)) {
+		for (const Cell* cell : cells) {
+			if (cell) {
+				cell->letter()->setPaused(m_paused);
+			}
+		}
+	}
+
+	if (m_paused) {
+		setCurrentWord(nullptr);
+	}
+
+	Q_EMIT pauseChanged();
+}
+
+//-----------------------------------------------------------------------------
+
+void Board::togglePaused()
+{
 	setPaused(!m_paused);
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::patternGenerated() {
+void Board::patternGenerated()
+{
 	m_words = m_pattern->solution();
 	QSize size = m_pattern->size();
 	setSceneRect(0, 0, size.width() * 34 + 2, size.height() * 34 + 34);
@@ -225,18 +273,18 @@ void Board::patternGenerated() {
 	for (int x = 0; x < size.width(); ++x) {
 		QList<Cell*> cells;
 		for (int y = 0; y < size.height(); ++y) {
-			cells.append(0);
+			cells.append(nullptr);
 		}
 		m_cells.append(cells);
 	}
 
-	foreach (Word* word, m_words) {
+	for (Word* word : std::as_const(m_words)) {
 		word->setBoard(this);
 		QList<QPoint> positions = word->positions();
 		for (int i = 0; i < positions.count(); ++i) {
 			const QPoint& pos = positions.at(i);
 			Cell* cell = m_cells.at(pos.x()).at(pos.y());
-			if (cell == 0) {
+			if (!cell) {
 				Letter* letter = new Letter(word->at(i), this);
 				addItem(letter);
 
@@ -245,13 +293,13 @@ void Board::patternGenerated() {
 				m_cells[pos.x()][pos.y()] = cell;
 			} else {
 				cell->letter()->setJoin();
-				cell->setWord(0);
+				cell->setWord(nullptr);
 			}
 		}
 	}
 
-	foreach (Word* word, m_words) {
-		emit wordAdded(word->toString());
+	for (Word* word : std::as_const(m_words)) {
+		Q_EMIT wordAdded(word->toString());
 		word->shuffle(m_pattern->words());
 	}
 
@@ -263,22 +311,42 @@ void Board::patternGenerated() {
 	}
 	saveGame();
 
-	emit started();
+	Q_EMIT started();
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::cleanUp() {
+void Board::generate(const QString& language, int pattern, int count, int length, unsigned int seed)
+{
+	cleanUp();
+	Q_EMIT loading();
+
+	m_wordlist->setLanguage(language);
+	m_pattern = Pattern::create(m_wordlist, pattern);
+	m_pattern->setCount(count);
+	m_pattern->setLength(length);
+	m_pattern->setSeed(seed);
+
+	connect(m_pattern, &Pattern::generated, this, &Board::patternGenerated);
+	m_pattern->start();
+}
+
+//-----------------------------------------------------------------------------
+
+void Board::cleanUp()
+{
 	delete m_pattern;
-	m_pattern = 0;
+	m_pattern = nullptr;
 	clear();
 	for (int i = 0; i < m_cells.count(); ++i) {
 		qDeleteAll(m_cells[i]);
 	}
 	m_cells.clear();
 	m_words.clear();
-	m_current_word = 0;
-	m_hint = 0;
+	m_current_word = nullptr;
+	m_hint = nullptr;
 	m_finished = false;
 	m_paused = false;
 }
+
+//-----------------------------------------------------------------------------
