@@ -1,27 +1,15 @@
-/***********************************************************************
- *
- * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2009 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "letter.h"
 
 #include "board.h"
 #include "cell.h"
 #include "word.h"
+#include "wordlist.h"
 
 #include <QCursor>
 #include <QFont>
@@ -30,14 +18,23 @@
 #include <QPainterPath>
 #include <QPen>
 
+//-----------------------------------------------------------------------------
+
 Letter::Letter(const QChar& character, Board* board)
-: m_board(board), m_character(character), m_cell(0), m_correct(false), m_movable(true), m_dragged(false), m_shadow(0) {
+	: m_board(board)
+	, m_character(character)
+	, m_cell(nullptr)
+	, m_correct(false)
+	, m_movable(true)
+	, m_dragged(false)
+	, m_shadow(nullptr)
+{
 	QPainterPath path;
 	path.addRoundedRect(0, 0, 32, 32, 5, 5);
 	setPath(path);
 	setCursor(Qt::OpenHandCursor);
 	setPen(Qt::NoPen);
-	setBrush(QColor("#bbbbbb"));
+	setBrush(QColor(0x90, 0x90, 0x90, 0x9d));
 	setFlag(QGraphicsItem::ItemIsMovable);
 	setZValue(1);
 
@@ -53,7 +50,8 @@ Letter::Letter(const QChar& character, Board* board)
 
 //-----------------------------------------------------------------------------
 
-void Letter::setCell(Cell* cell) {
+void Letter::setCell(Cell* cell)
+{
 	m_cell = cell;
 	QGraphicsItem* item = m_shadow ? m_shadow : this;
 	item->setPos(m_cell->position().x() * 34 + 2, m_cell->position().y() * 34 + 34);
@@ -61,34 +59,39 @@ void Letter::setCell(Cell* cell) {
 
 //-----------------------------------------------------------------------------
 
-void Letter::setCorrect() {
+void Letter::setCorrect()
+{
 	setFlag(QGraphicsItem::ItemIsMovable, false);
-	setBrush(QColor("#008c00"));
+	setBrush(QColor(0, 0x8c, 0));
 	setCursor(Qt::PointingHandCursor);
+	setText(m_character);
 	m_correct = true;
 	m_movable = false;
 }
 
 //-----------------------------------------------------------------------------
 
-void Letter::setHighlight(bool highlight) {
+void Letter::setHighlight(bool highlight)
+{
 	if (m_movable) {
-		setBrush(highlight ? QColor("#0057ae") : QColor("#bbbbbb"));
+		setBrush(highlight ? QColor(0, 0x57, 0xae) : QColor(0x90, 0x90, 0x90, 0x9d));
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-void Letter::setJoin() {
+void Letter::setJoin()
+{
 	setFlag(QGraphicsItem::ItemIsMovable, false);
-	setBrush(QColor("#555555"));
+	setBrush(QColor(0, 0, 0, 0xaa));
 	setCursor(Qt::ArrowCursor);
 	m_movable = false;
 }
 
 //-----------------------------------------------------------------------------
 
-void Letter::setPaused(bool paused) {
+void Letter::setPaused(bool paused)
+{
 	if (m_movable) {
 		setText(!paused ? m_character : '?');
 	}
@@ -96,7 +99,22 @@ void Letter::setPaused(bool paused) {
 
 //-----------------------------------------------------------------------------
 
-void Letter::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
+void Letter::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event)
+{
+	if (!m_movable && m_correct) {
+		Word* word = m_cell->word();
+		if (word) {
+			word->click();
+		}
+	}
+
+	QGraphicsPathItem::mouseDoubleClickEvent(event);
+}
+
+//-----------------------------------------------------------------------------
+
+void Letter::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
+{
 	if (m_dragged == false) {
 		event->accept();
 		return;
@@ -109,18 +127,18 @@ void Letter::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 	}
 
 	QPoint pos = (((mapToScene(boundingRect().center()) - QPoint(2.f, 34.f)) / 34.f) - QPointF(0.5f, 0.5f)).toPoint();
-	Cell* cell = 0;
+	Cell* cell = nullptr;
 	if (m_cell->word()->orientation() == Qt::Horizontal) {
 		cell = m_board->cell(pos.x(), m_cell->position().y());
 	} else {
 		cell = m_board->cell(m_cell->position().x(), pos.y());
 	}
-	if (cell == 0 || cell == m_cell || cell->word() != m_cell->word()) {
+	if (!cell || cell == m_cell || cell->word() != m_cell->word()) {
 		return;
 	}
 
 	Letter* letter = cell->letter();
-	if (letter == 0 || letter->m_movable == false) {
+	if (!letter || letter->m_movable == false) {
 		return;
 	}
 	m_cell->setLetter(letter);
@@ -129,7 +147,8 @@ void Letter::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 
 //-----------------------------------------------------------------------------
 
-void Letter::mousePressEvent(QGraphicsSceneMouseEvent* event) {
+void Letter::mousePressEvent(QGraphicsSceneMouseEvent* event)
+{
 	if (event->button() != Qt::LeftButton) {
 		event->ignore();
 		return;
@@ -139,9 +158,7 @@ void Letter::mousePressEvent(QGraphicsSceneMouseEvent* event) {
 	if (!m_movable) {
 		if (m_correct) {
 			Word* word = m_cell->word();
-			if (word) {
-				word->click();
-			} else {
+			if (!word) {
 				m_board->click("");
 			}
 		}
@@ -153,7 +170,7 @@ void Letter::mousePressEvent(QGraphicsSceneMouseEvent* event) {
 	m_board->setPaused(false);
 	m_board->setCurrentWord(m_cell->word());
 
-	m_shadow = scene()->addPath(path(), Qt::NoPen, QColor("#a5c1e4"));
+	m_shadow = scene()->addPath(path(), Qt::NoPen, QColor(0xa5, 0xc1, 0xe4));
 	m_shadow->setPos(pos());
 
 	QGraphicsPathItem::mousePressEvent(event);
@@ -161,7 +178,8 @@ void Letter::mousePressEvent(QGraphicsSceneMouseEvent* event) {
 
 //-----------------------------------------------------------------------------
 
-void Letter::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
+void Letter::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
+{
 	if (event->button() != Qt::LeftButton) {
 		event->ignore();
 		return;
@@ -179,7 +197,7 @@ void Letter::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
 
 	setPos(m_shadow->pos());
 	delete m_shadow;
-	m_shadow = 0;
+	m_shadow = nullptr;
 
 	m_cell->word()->check();
 
@@ -188,7 +206,33 @@ void Letter::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
 
 //-----------------------------------------------------------------------------
 
-void Letter::setText(const QChar& character) {
-	m_text->setText(character);
+void Letter::setText(const QChar& character)
+{
+	if (m_correct || (m_board->words()->language() != "he")) {
+		m_text->setText(character);
+	} else {
+		switch (character.unicode()) {
+		case u'ך':
+			m_text->setText(QChar(u'כ'));
+			break;
+		case u'ם':
+			m_text->setText(QChar(u'מ'));
+			break;
+		case u'ן':
+			m_text->setText(QChar(u'נ'));
+			break;
+		case u'ף':
+			m_text->setText(QChar(u'פ'));
+			break;
+		case u'ץ':
+			m_text->setText(QChar(u'צ'));
+			break;
+		default:
+			m_text->setText(character);
+			break;
+		}
+	}
 	m_text->setPos(boundingRect().center() - m_text->boundingRect().center());
 }
+
+//-----------------------------------------------------------------------------
