@@ -1,21 +1,8 @@
-/***********************************************************************
- *
- * Copyright (C) 2010, 2011, 2012, 2014, 2015 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2010 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "locale_dialog.h"
 
@@ -43,8 +30,8 @@ QString LocaleDialog::m_appname;
 
 //-----------------------------------------------------------------------------
 
-LocaleDialog::LocaleDialog(QWidget* parent) :
-	QDialog(parent, Qt::WindowTitleHint | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowSystemMenuHint | Qt::WindowCloseButtonHint)
+LocaleDialog::LocaleDialog(QWidget* parent)
+	: QDialog(parent, Qt::WindowTitleHint | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowSystemMenuHint | Qt::WindowCloseButtonHint)
 {
 	QString title = parent ? parent->window()->windowTitle() : QString();
 	setWindowTitle(!title.isEmpty() ? title : QCoreApplication::applicationName());
@@ -53,7 +40,7 @@ LocaleDialog::LocaleDialog(QWidget* parent) :
 
 	m_translations = new QComboBox(this);
 	m_translations->addItem(tr("<System Language>"));
-	QStringList translations = findTranslations();
+	const QStringList translations = findTranslations();
 	for (QString translation : translations) {
 		if (translation.startsWith("qt")) {
 			continue;
@@ -77,74 +64,61 @@ LocaleDialog::LocaleDialog(QWidget* parent) :
 
 //-----------------------------------------------------------------------------
 
-void LocaleDialog::loadTranslator(const QString& name, const QStringList& datadirs)
+void LocaleDialog::loadTranslator(const QString& name, const QString& datadir)
 {
 	m_appname = name;
 
 	// Find translator path
-	QStringList paths = datadirs;
-	if (paths.isEmpty()) {
-		QString appdir = QCoreApplication::applicationDirPath();
-		paths.append(appdir);
-		paths.append(appdir + "/../share/" + QCoreApplication::applicationName().toLower());
-		paths.append(appdir + "/../Resources");
-	}
-	for (const QString& path : paths) {
-		if (QFile::exists(path + "/translations/")) {
-			m_path = path + "/translations/";
-			break;
-		}
-	}
+	m_path = datadir + "/translations/";
 
 	// Find current locale
 	m_current = QSettings().value("Locale/Language").toString();
-	QString current = !m_current.isEmpty() ? m_current : QLocale::system().name();
-	QStringList translations = findTranslations();
-	if (!translations.contains(m_appname + current)) {
-		current = current.left(2);
-		if (!translations.contains(m_appname + current)) {
-			current.clear();
-		}
-	}
-	if (!current.isEmpty()) {
-		QLocale::setDefault(current);
-	} else {
-		current = "en";
+	if (!m_current.isEmpty()) {
+		QLocale::setDefault(QLocale(m_current));
 	}
+	const QString locale = QLocale().name();
 
 	// Load translators
-	static QTranslator qt_translator;
-	if (translations.contains("qt_" + current) || translations.contains("qt_" + current.left(2))) {
-		qt_translator.load("qt_" + current, m_path);
-	} else {
-		qt_translator.load("qt_" + current, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
-	}
-	QCoreApplication::installTranslator(&qt_translator);
-
 	static QTranslator translator;
-	translator.load(m_appname + current, m_path);
-	QCoreApplication::installTranslator(&translator);
+	if (translator.load(m_appname + locale, m_path)) {
+		QCoreApplication::installTranslator(&translator);
 
-	// Work around bug in Qt 5 where text direction is not loaded
-	QGuiApplication::setLayoutDirection(QLocale(current).textDirection());
+		const QString path = QLibraryInfo::path(QLibraryInfo::TranslationsPath);
+
+		static QTranslator qtbase_translator;
+		if (qtbase_translator.load("qtbase_" + locale, m_path) || qtbase_translator.load("qtbase_" + locale, path)) {
+			QCoreApplication::installTranslator(&qtbase_translator);
+		}
+
+		static QTranslator qt_translator;
+		if (qt_translator.load("qt_" + locale, m_path) || qt_translator.load("qt_" + locale, path)) {
+			QCoreApplication::installTranslator(&qt_translator);
+		}
+	}
 }
 
 //-----------------------------------------------------------------------------
 
 QString LocaleDialog::languageName(const QString& language)
 {
-	QString lang_code = language.left(5);
-	QLocale locale(lang_code);
 	QString name;
-	if (lang_code.length() > 2) {
-		if (locale.name() == lang_code) {
-			name = locale.nativeLanguageName() + " (" + locale.nativeCountryName() + ")";
+	const QLocale locale(language);
+	if (language.contains('_')) {
+		if (locale.name() == language) {
+			name = locale.nativeLanguageName() + " (" + locale.nativeTerritoryName() + ")";
 		} else {
 			name = locale.nativeLanguageName() + " (" + language + ")";
 		}
 	} else {
 		name = locale.nativeLanguageName();
 	}
+	if (name.isEmpty() || name == "C") {
+		if (language == "eo") {
+			name = "Esperanto";
+		} else {
+			name = language;
+		}
+	}
 	if (locale.textDirection() == Qt::RightToLeft) {
 		name.prepend(QChar(0x202b));
 	}
